/*
** This software is Copyright (c) 1989, 1990, 1991, 1992, 1993
**                                1994, 1995 by Kent Landfield
**
** Permission is hereby granted to copy, distribute or otherwise 
** use any part of this package as long as you do not try to make 
** money from it or pretend that you wrote it.  This copyright 
** notice must be maintained in any copy made.
**
** This file contains functions to maintain the .archived file in the
** newsgroup's archive BASEDIR directory. The .archived file is used to 
** assure that articles are not archived over and over and that the 
** administrator does not need to wait until the articles have
** expired before they are added to the archive. 
**
** The .archived file record formats for the various methods of archiving
** are different so that the system can check to see if a record is still
** valid because the article has not expired from the system.
**
** FROM_NNTP:
**    article's-message_id	nntp_server	path-to-archived-file
**
** FROM_DISK:
**    article's-message_id	filename		path-to-archived-file
**
** FROM_BATCHFILE:
** FROM_STDIN:
** FROM_NAME:
**    article's-message_id	E:date-to-expire-record	path-to-archived-file
*/

#if !defined(lint) && !defined(CODECENTER)
static char SID[] = "@(#)record_arc.c	2.3 1/21/95";
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
# ifdef LOG_DBM
#include <sys/file.h>
#include <ndbm.h>
# endif /*LOG_DBM*/
#include "cfg.h"

static char work[BUFSIZ];
static int len;
static FILE *archived_fp;

#ifndef LOG_DBM
#define SECS_PER_DAY 86400L            /* total seconds per day  */
#endif

int fclose();
int sscanf();
int strlen();
int strncmp();
FILE *fopen();
FILE *efopen();

#ifdef NNTP
# ifndef LOG_DBM
    extern char *nntp_server;
# endif /*LOG_DBM*/
  extern int verbose;
#endif /* NNTP */

extern int retrieve;
extern int test;
extern struct group_archive *newsgrp;
extern char newsgroup_directory[];

/*
** needs_to_be_archived:
**
** int needs_to_be_archived(message_id) char *message_id; { return(0); }
**
** This function is used to determine if the message-id
** passed to it requires archiving. If the message-id is
** found in the .archived file, it has been previously
** archived and does not need to be now. If it is not
** found in the .archived file, it needs to be archived.
*/

int needs_to_be_archived(message_id)
    char *message_id;
{
    char *get_archived_rec();

    if (get_archived_rec(message_id) == NULL)
	return(1);
#ifdef NNTP
    if (verbose && retrieve == FROM_NNTP) 
        (void) fprintf(logfp,"Skipping, %s, previously archived.\n",message_id);
#endif /*NNTP*/
    return(0);
}

#ifndef LOG_DBM
/*
** get_archived_rec:
**
** get_archived_rec(message_id) char *message_id; { return(work); }
**
** get_archived_rec is used to determine if the message_id
** passed to it has been archived. If its message_id is found
** in the .archived file, it has been previously archived
** and its archive information record is  returned to the
** caling routine. Else it returns NULL.
*/

char *get_archived_rec(message_id)
    char *message_id;
{
    /*
    ** If the .archived file is missing, it must be
    ** the first time that it is to be archived.
    */
    if ((archived_fp = fopen(newsgrp->arc_done,"r")) == NULL) 
	return(NULL);

    len = strlen(message_id);
 
    while (fgets(work, sizeof work, archived_fp) != NULL) {
        if (*work == '#')
            continue;
        if ((strncmp(work,message_id,len) == 0) && (work[len] == ' ')) {
            (void) fclose(archived_fp);
            *(work+(strlen(work)-1)) = '\0';
            return(work);
        }
    }
    (void) fclose(archived_fp);
    return(NULL);
}


/*
** write_archived:
**
** write_archived(filename,message_id,path) char *filename,*message_id,*path; {}
**
** This function is used to write a message_id and its archived location
** to the .archived file in the newsgroup's archive BASEDIR
** directory since we do not want it rearchived tomorrow.
*/

void write_archived(filename, message_id, path)
    char *filename;
    char *message_id;
    char *path;
{
#ifdef NOTDEF
    char *expire_date();
#endif

    FILE *fp;
    struct stat sb;

    /* 
    ** If in test mode no actual operations are 
    ** to be done...
    */
    if (test)
        return;

    if ((stat(newsgrp->arc_done ,&sb) != 0)) 
        if (mkparents(newsgrp->arc_done) == -1)
            return;          /* unable to build parent directories */

    /*
    ** open the file and append the record
    */

    fp = efopen(newsgrp->arc_done,"a");

    if (retrieve == FROM_DISK)
        (void) fprintf(fp,"%s %s %s\n",message_id, filename, path);
#ifdef NNTP
    else if (retrieve == FROM_NNTP) 
        (void) fprintf(fp,"%s %s %s\n",message_id,nntp_server,path);
#endif /*NNTP*/
#ifdef NOTDEF
    else 
        (void) fprintf(fp,"%s E%s %s\n",message_id, expire_date(), path);
#endif
    (void) fclose(fp);
    return;
}


/*
** remove_expired:
**
** This function is used to maintain the .archived file in the
** newsgroup's archive BASEDIR directory. This file is used to assure
** that files are not archived over and over and that the 
** administrator does not need to wait until the files have
** expired before they are added to the archive. 
** The .archived file stores a list of filenames of files that have been 
** archived already. The remove_expired function removes filenames 
** from the .archived file that no longer exist. In this manner the 
** file is self maintaining. Expired entries are removed.
*/

void remove_expired()
{
    char *strchr();
#ifdef NNTP
    void close_server();
    int stat_article();

    char *nntpsvr;
    int rnum;
#endif /* NNTP */
    int num_entries;
    char news_file[128];
    char message_id[256];
    char fl_loc[MAXNAMLEN];
    char tmp_loc[MAXNAMLEN];
    FILE *tmp_archived;
    struct stat stbuf;

    /* Is this being run as a test ? */

    if (test)
        return;

    /* Is this the first time this group is to be archived ? */

    if (stat(newsgrp->arc_done, &stbuf) < 0) 
        return;
    
    /* open the necessary files, creating a temp file */
    (void) sprintf(tmp_loc,"%s/.arch_tmp", newsgrp->location);

    tmp_archived = efopen(tmp_loc,"w");
    archived_fp = efopen(newsgrp->arc_done,"r");

    /* 
    ** for every line in the .archived file,
    **    check it comment line. If so, save it in the temp file
    **    strip the string of any blanks or tabs inadvertantly 
    **      added by admin in manual mode (editor)
    **    check to see if the file still exists.
    **    if so, write filename to the temp file
    ** endfor
    */

#ifdef NNTP
    rnum = 0;
#endif /* NNTP */

    while (fgets(work, sizeof work, archived_fp) != NULL) {
        if (*work == '#') {      /* maintain comments in .archived */
            (void) fprintf(tmp_archived,"%s",work);
            continue;
        }
        num_entries = sscanf(work,"%s %s %s",message_id,news_file,fl_loc);
        if (num_entries == 3) {
            /* 
            ** If the file is being archived from 
            ** the local disk then check it out.
            */
#ifdef NOTDEF
fprintf(stderr,"News_file == %s\n", news_file);
#endif NOTDEF
            if (isdigit(*news_file)) {
                (void) sprintf(fl_loc,"%s/%s",newsgroup_directory,news_file);
                if (stat(fl_loc, &stbuf) == 0) 
                    (void) fprintf(tmp_archived,"%s",work);
            }
#ifdef NOTDEF
            else if (*news_file == 'E') {
                if (not_expired(news_file+1))
                    (void) fprintf(tmp_archived,"%s",work);
            }
#endif

#ifdef NNTP
            else if (strchr(news_file,'.') != NULL) {
                /*
                ** The article was archived from an NNTP server
                ** and the format of the news_file name is
                **    host.nntp.server (full domainame)
                */
                nntpsvr = news_file;
                if (stat_article(nntpsvr,message_id,rnum++) == RETRIEVED)
                    (void) fprintf(tmp_archived,"%s",work);
            }
#endif /* NNTP */
            else {
                (void) fprintf(logfp,"unknown .archived record format, [%s], putting it back.\n",work);
                /* Ignore and put back for now... */
                (void) fprintf(tmp_archived,"%s",work);
            }
        }
        else  /* old entry... just put back as an Expire record */
#ifdef NOTDEF
            (void) fprintf(fp,"%s E%s %s\n",message_id,expire_date(),news_file);
#else
            /* Ignore and put back for now... */
            (void) fprintf(tmp_archived,"%s",work);
#endif /* NOTDEF */
    }

    /*
    ** Close both files and rename the temp 
    ** file to the new .archived file.
    */

#ifdef NNTP 
    close_server();
#endif /* NNTP */
    (void) fclose(tmp_archived);
    (void) fclose(archived_fp);
    (void) rename(tmp_loc,newsgrp->arc_done);
    return;
}

#ifdef NOTDEF
#include <time.h>

/**********************************************************************
**                                                                    
**  Function:       expire_date                                          
**                                                                  
**  Abstract:   This function converts unix time (long integer) into
**              the audit format (yymmddhhmmss).                    
**  Returns:
**              wk - address of a static string containing the      
**                   audit time string. Subsequent calls overwrite  
**                   the previous contents of wk.                   
**                                                                  
**********************************************************************/

char *expire_date()
{
   struct tm *gmtime();         /* libc declaration         */
   char *strcat();              /* libc declaration         */
   long time();                 /* libc declaration         */
   struct tm *gmt;              /* tm structure pointer     */
   long clk;                    /* time in seconds variable */
   static char wkstr[13];       /* return string            */

   /* get the current time */

   clk = time((long *) 0);

   clk += (SECS_PER_DAY * DAYS_TO_EXPIRE);

   /* convert the time to current zulu (gmt) time */

   gmt = localtime(&clk);

   /* build a string containing the proper format */
   /* and zero padded if single digit returned.   */

   (void) sprintf(wkstr, "%02d%02d%02d%02d%02d%02d",
       gmt->tm_year, gmt->tm_mon + 1, gmt->tm_mday, 
       gmt->tm_hour, gmt->tm_min, gmt->tm_sec);

   wkstr[12] = '\0'; /* terminate the string and return it */
   return(wkstr);
}

int not_expired(s)
char *s;
{
    long time();
    long date_to_long();
 
    long clk;
    long val;
    static int no_time = 1;
 
    if (no_time) {
       clk = time((long *)0);
       no_time = 0;
    }

    val = date_to_long(s);

    if (val > clk)
        return(1);
    return(0);
}
#endif 
#else /*LOG_DBM*/

/*
** get_archived_rec:
**
** get_archived_rec(message_id) char *message_id; { return(work); }
**
** get_archived_rec is used to determine if the message_id
** passed to it has been archived. If its message_id is found
** in the .archived file, it has been previously archived
** and its archive information record is  returned to the
** caling routine. Else it returns NULL.
*/
 
char *get_archived_rec(message_id)
    char *message_id;
{
    char *strncpy();
    datum key, content;

    /*
    ** If the .archived file is missing, it must be
    ** the first time that it is to be archived.
    */

    if (archived_db == NULL)
       archived_db = dbm_open(archived_log, O_CREAT | O_RDWR, 0644);
    if (archived_db == NULL) return(NULL); /* error opening DB file, do later */
    len = strlen(message_id);

    key.dptr = message_id;
    key.dsize = strlen(message_id) + 1;
    content = dbm_fetch(archived_db, key);
    if (content.dptr == NULL)
       return(NULL);
    else {
       (void) strncpy(work, content.dptr, content.dsize);
       work[content.dsize] = NULL;
       return(work);
    }
}

/*
** write_archived:
**
** write_archived(message_id, path) char *message_id, path; {}
**
** This function is used to write a message_id and its archived location
** to the .archived file in the newsgroup's archive BASEDIR
** directory since we do not want it rearchived tomorrow.
*/
 
void write_archived(message_id, path)
    char *message_id;
    char *path;
{
    datum key, content;
    struct stat sb;
 
    /*
    ** If in test mode no actual operations are
    ** to be done...
    */
    if (test)
        return;
 
    if ((stat(archived_log, &sb) != 0))
        if (mkparents(archived_log) == -1)
            return;          /* unable to build parent directories */
 
    /*
    ** open the file and append the record
    */
 
    if (archived_db == NULL)
       archived_db = dbm_open(archived_log, O_CREAT | O_RDWR, 0644);
    key.dptr = message_id;
    key.dsize = strlen(message_id) + 1;
    content.dptr = path;
    content.dsize = strlen(path) + 1;
    (void) dbm_store(archived_db, key, content, DBM_INSERT);
    return;
}      

#endif /*LOG_DBM*/
